# -*- coding: utf-8 -*-
"""
A list of Romanian counties as `choices` in a formfield.

This exists as a standalone file so that it's only imported into memory when
explicitly needed.
"""

COUNTIES_CHOICES = (
    ('AB', u'Alba'),
    ('AR', u'Arad'),
    ('AG', u'Argeş'),
    ('BC', u'Bacău'),
    ('BH', u'Bihor'),
    ('BN', u'Bistriţa-Năsăud'),
    ('BT', u'Botoşani'),
    ('BV', u'Braşov'),
    ('BR', u'Brăila'),
    ('B',  u'Bucureşti'),
    ('BZ', u'Buzău'),
    ('CS', u'Caraş-Severin'),
    ('CL', u'Călăraşi'),
    ('CJ', u'Cluj'),
    ('CT', u'Constanţa'),
    ('CV', u'Covasna'),
    ('DB', u'Dâmboviţa'),
    ('DJ', u'Dolj'),
    ('GL', u'Galaţi'),
    ('GR', u'Giurgiu'),
    ('GJ', u'Gorj'),
    ('HR', u'Harghita'),
    ('HD', u'Hunedoara'),
    ('IL', u'Ialomiţa'),
    ('IS', u'Iaşi'),
    ('IF', u'Ilfov'),
    ('MM', u'Maramureş'),
    ('MH', u'Mehedinţi'),
    ('MS', u'Mureş'),
    ('NT', u'Neamţ'),
    ('OT', u'Olt'),
    ('PH', u'Prahova'),
    ('SM', u'Satu Mare'),
    ('SJ', u'Sălaj'),
    ('SB', u'Sibiu'),
    ('SV', u'Suceava'),
    ('TR', u'Teleorman'),
    ('TM', u'Timiş'),
    ('TL', u'Tulcea'),
    ('VS', u'Vaslui'),
    ('VL', u'Vâlcea'),
    ('VN', u'Vrancea'),
)
